<?php
namespace EMS\PortalFuncionario\Geral\Services;

use Zeedhi\Framework\Controller\Crud;
use Doctrine\ORM\EntityManager;

use EMS\PortalFuncionario\Geral\Repository\ApuracaoRepository;
use EMS\PortalFuncionario\Util\{ DateUtil, HcmServices };

class ApuracaoService extends Crud {

	public function __construct(
        EntityManager       $entityManager,
        ApuracaoRepository  $apuracaoRepository,
        HcmServices         $hcmServices
    ) {
		$this->entityManager        = $entityManager;
        $this->apuracaoRepository   = $apuracaoRepository;
        $this->hcmServices          = $hcmServices;
    }

    public function getPeriodoApuracao(
        string $nrvinculom,
        string $nrorg
    ) : array {
        try {
            return $this->apuracaoRepository->getPeriodoApuracao(
                $nrvinculom,
                $nrorg
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getApuracaoPorData(
        string $nrvinculom,
        string $nrorg,
        string $nrperiodoapuracao = null,
        string $dtapuracaoponto = null,
        string $dtmescompetencia = null
    ) { 
        if (empty($nrperiodoapuracao)) {
            $nrperiodoapuracao = $this->apuracaoRepository->getPeriodoApuracaoAtual(
                $nrvinculom,
                $nrorg
            );
            $nrperiodoapuracao = $nrperiodoapuracao['NRPERIODOAPURACAO'];
        }

        if (empty($dtapuracaoponto)) {
            $dtapuracaoponto = $this->getUltimoDiaApurado(
                $nrvinculom,
                $nrorg,
                $nrperiodoapuracao
            );

            if (!$dtapuracaoponto) {
                throw new \Exception('Não há apurações realizadas nesse período.');
            }
        }

        if (empty($dtmescompetencia)) {
            $dtmescompetencia = DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataAtual())->format(DateUtil::FORMATO_BRASILEIRO);
        }

        return $this->apuracaoRepository->getApuracaoPorData(
            $nrvinculom,
            $nrorg,
            $nrperiodoapuracao,
            $dtapuracaoponto,
            $dtmescompetencia
        );
    }

    public function getSaldosPeriodoApuracao(
        string $nrvinculom,
        string $nrorg,
        string $nrperiodoapuracao = null,
        string $dtmescompetencia = null
    ) {
        if (empty($nrperiodoapuracao)) {
            $nrperiodoapuracao = $this->apuracaoRepository->getPeriodoApuracaoAtual(
                $nrvinculom,
                $nrorg
            );
            $nrperiodoapuracao = $nrperiodoapuracao['NRPERIODOAPURACAO'];
        }

        if (empty($dtmescompetencia)) {
            $dtmescompetencia = DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataAtual())->format(DateUtil::FORMATO_BRASILEIRO);
        }

        $resultado = $this->apuracaoRepository->getSaldosPeriodoApuracao(
            $nrvinculom,
            $nrorg,
            $nrperiodoapuracao,
            $dtmescompetencia
        );

        $resultado['NRPERIODOAPURACAO'] = $nrperiodoapuracao;
        return $resultado;
    }

    /**
     * Rotina similar a getApuracaoPorData, porém destinada a retornar o saldo de banco de horas apenas
     */
    public function getResumoSaldoBanco(
        string $nrvinculom,
        string $nrorg,
        string $nrperiodoapuracao = null,
        string $dtapuracaoponto = null,
        string $dtmescompetencia = null
    ) {
        $saldoBancoVariavel = 'SALDO_BH_ATUAL';

        $data = $this->getApuracaoPorData(
            $nrvinculom,
            $nrorg,
            $nrperiodoapuracao,
            $dtapuracaoponto,
            $dtmescompetencia
        );

        $saldoBanco = empty($data[$saldoBancoVariavel]) ? 0 : $data[$saldoBancoVariavel];

        return array($saldoBanco);
    }

    public function getUltimoDiaApurado(
        string $nrvinculom,
        string $nrorg,
        string $nrperiodoapuracao
    ) {

        $ultimoDiaApurado = $this->apuracaoRepository->getUltimoDiaApurado(
            $nrvinculom,
            $nrorg,
            $nrperiodoapuracao
        );

        if (!empty($ultimoDiaApurado) && !empty($ultimoDiaApurado["DTAPURACAOSITUFREQ"])) {
            $dtUltimoDiaApurado = new \DateTime($ultimoDiaApurado["DTAPURACAOSITUFREQ"]);
            return $dtUltimoDiaApurado->format('d/m/Y');
        }

        return false;

    }

    public function apurarDias(
        string  $nrorg,
        string  $nrvinculom,
        Array   $dias
    ) {
        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'        => $nrvinculom,
                'NRORG'             => $nrorg,
                'DIAS'              => $dias
            ]
        ];

        $result = $this->hcmServices->send('apurarDias', $params, $nrorg, "POST");

        return $result['dataset']['apurarDias'];
    }

    public function apurarPeriodo(
        string  $nrorg,
        string  $nrvinculom,
        string  $nrperiodoapuracao
    ) {
        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'        => $nrvinculom,
                'NRORG'             => $nrorg,
                'NRPERIODOAPURACAO' => $nrperiodoapuracao
            ]
        ];

        $result = $this->hcmServices->send('apurarPeriodo', $params, $nrorg, "POST");

        return $result['dataset']['apurarPeriodo'];
    }

    public function verificaDiaTemResumo(
        int    $nrOrg,
        int    $nrVinculo,
        string $date
    ): bool {
        $route = 'verificaDiaTemResumo';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM' => $nrVinculo,
                'DATE'       => $date
            ]
        ];

        $result = $this->hcmServices->send($route, $params, $nrOrg);
        $temResumo = $result['dataset'][$route]['temResumo'];

        return $temResumo;
    }

    public function apurarPeriodoFuncionario(
        int    $nrOrg,
        int    $nrVinculo,
        int    $nrPeriodo
    ): bool {
        $route = 'apurarPeriodoFuncionario';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'        => $nrVinculo,
                'NRPERIODOAPURACAO' => $nrPeriodo,
            ]
        ];

        $result = $this->hcmServices->send($route, $params, $nrOrg);
        $success = $result['dataset'][$route]['success'];

        return $success;
    }
}